﻿/*
 Mediator - PureMVC
 */
package view {
	import flash.display.Stage;
	
	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;
	import view.*;
	import view.components.Card;
	
	
	/**
	 * A Mediator
	 */
	public class CardMediator extends Mediator implements IMediator {
		private var _card:Card;
	
		// Cannonical name of the Mediator
		public static const NAME:String = "CardMediator";
		
		public function CardMediator(viewComponent:Object) {
			// pass the viewComponent to the superclass where 
			// it will be stored in the inherited viewComponent property
			super(NAME, viewComponent);
		}
		
		override public function onRegister():void {
			_card = new Card();
			stage.addChild(_card);
		}

		/**
		 * Get the Mediator name.
		 * <P>
		 * Called by the framework to get the name of this
		 * mediator. If there is only one instance, we may
		 * define it in a constant and return it here. If
		 * there are multiple instances, this method must
		 * return the unique name of this instance.</P>
		 * 
		 * @return String the Mediator name
		 */
		override public function getMediatorName():String {
			return CardMediator.NAME;
		}
        
		/**
		 * List all notifications this Mediator is interested in.
		 * <P>
		 * Automatically called by the framework when the mediator
		 * is registered with the view.</P>
		 * 
		 * @return Array the list of Nofitication names
		 */
		override public function listNotificationInterests():Array {
			return [];
		}

		/**
		 * Handle all notifications this Mediator is interested in.
		 * <P>
		 * Called by the framework when a notification is sent that
		 * this mediator expressed an interest in when registered
		 * (see <code>listNotificationInterests</code>.</P>
		 * 
		 * @param INotification a notification 
		 */
		override public function handleNotification(note:INotification):void {
			switch (note.getName()) {           
				default:
					break;		
			}
		}
		
		public function rotateCard(num:uint):void {
			trace(">> rotateCard : " + num);
			_card.rotateCard(num);
			sendNotification(ApplicationFacade.SUB_UPDATE, num);
		}
		
		protected function get stage():Stage {
			return viewComponent as Stage;
		}

	}
}
